@@ -34,7 +34,7 @@ from psutil import virtual_memory as vmem
 
 import pandas as pd
 import json
-from npyx.io import chan_map, read_metadata
+from npyx.inout import chan_map, read_metadata
 from sklearn.decomposition import PCA
 from sklearn.preprocessing import StandardScaler
 from npyx.corr import (ccg, StarkAbeles2009_ccg_significance, ccg_sig_stack, gen_sfc, scaled_acg)
@@ -227,8 +227,8 @@ for i, ds in list(recs.items())[:]:
     print(ds['units'])
     spike_clusters= np.load(Path(ds['dp'], 'spike_clusters.npy'))
 
-    cell_type = str(ds['dp']).split('/')[-2]
-    rec_name = str(ds['dp']).split('/')[-1]
+    cell_type = Path(ds['dp']).parts[-2]
+    rec_name = Path(ds['dp']).parts[-1]
     features_filename = rec_name + '_' + cell_type + '.csv'
     features_file = features_folder / features_filename
     rec_feat = []
@@ -978,7 +978,7 @@ def chan_spread(all_wav, chan_path, unit, n_chans = 20, chan_spread_dist = 25.6)
 
 
 #    if chan_spread_dist == 25.6: chan_spread_dist = 25.61249695
-    vals_at_25 = sort_dist_p2p[:,1][np.isclose(sort_dist_p2p[:,0], chan_spread_dist )]
+    vals_at_25 = sort_dist_p2p[:,1][np.round(sort_dist_p2p[:,0])==np.round(chan_spread_dist)]
     spread =np.max(vals_at_25)
 
     return max_chan, dists, p2p, dist_p2p, sort_dist_p2p, spread
@@ -1415,7 +1415,7 @@ def temp_feat(dp, units, use_or_operator = True, use_consecutive = False):
     all_ft_list = []
     for unit in units:
 
-        unit_spikes = trn_filtered(dp, unit, use_or_operator = use_or_operator, use_consecutive = use_consecutive)
+        unit_spikes, good_spikes_m = trn_filtered(dp, unit, use_or_operator = use_or_operator, use_consecutive = use_consecutive)
         if len(unit_spikes) >1:
             all_ft_list.append(temporal_features(dp,unit_spikes, unit))
         else:
@@ -1579,8 +1579,8 @@ def gen_ss_cs(recs_fn, show = False):
                 new_json[i]['ss'] = ss_list.tolist()
                 new_json[i]['cs'] = cs_list.tolist()
     #        breakpoint()
-        except AssertionError: print(f"Recording {dp.split('/')[-1]} can't be processed")
-        except IndexError: print(f"Recording {dp.split('/')[-1]} can't be processed")
+        except AssertionError: print(f"Recording {Path(dp).parts[-1]} can't be processed")
+        except IndexError: print(f"Recording {Path(dp).parts[-1]} can't be processed")
 
     with open(recs_fn, 'w') as json_handle:
         json.dump(new_json, json_handle, indent=2)
@@ -1631,8 +1631,8 @@ def process_all(recs_fn, show = False, again = False):
 #        print(ds['units'])
         spike_clusters= np.load(Path(ds['dp'], 'spike_clusters.npy'))
 
-        cell_type = str(ds['dp']).split('/')[-2]
-        rec_name = str(ds['dp']).split('/')[-1]
+        cell_type = Path(ds['dp']).parts[-2]
+        rec_name = Path(ds['dp']).parts[-1]
         features_filename = rec_name + '_' + cell_type + '.csv'
         features_file = features_folder / features_filename
         rec_feat = []
@@ -1711,8 +1711,8 @@ def process_all(recs_fn, show = False, again = False):
 
         # get all the good units
 
-        cell_type = str(ds['dp']).split('/')[-2]
-        rec_name = str(ds['dp']).split('/')[-1]
+        cell_type = Path(ds['dp']).parts[-2]
+        rec_name = Path(ds['dp']).parts[-1]
         features_filename = rec_name + '_' + cell_type + '.csv'
         features_file = features_folder / features_filename
 
@@ -1823,8 +1823,8 @@ def process_all(recs_fn, show = False, again = False):
         # get all the good units
 
 
-        cell_type = str(ds['dp']).split('/')[-2]
-        rec_name = str(ds['dp']).split('/')[-1]
+        cell_type = Path(ds['dp']).parts[-2]
+        rec_name = Path(ds['dp']).parts[-1]
         features_filename = rec_name + '_' + cell_type + '.csv'
     #    print(features_filename)
         features_file = features_folder / features_filename
@@ -1882,8 +1882,8 @@ def process_all(recs_fn, show = False, again = False):
 
         # get all the good units
 
-        cell_type = str(ds['dp']).split('/')[-2]
-        rec_name = str(ds['dp']).split('/')[-1]
+        cell_type = Path(ds['dp']).parts[-2]
+        rec_name = Path(ds['dp']).parts[-1]
         features_filename = rec_name + '_' + cell_type + '.csv'
         features_file = features_folder / features_filename
 
